==> pan-0.6.4.tar.gz: pan-0.6.4/pan/article.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include "pan.h"

void
article_add_flag (
	article_data* adata,
	int flag,
	gboolean sync )
{
	adata->state |= flag;
	article_update ( adata, sync );
}


void
article_remove_flag (
	article_data* adata,
	int flag,
	gboolean sync )
{
	adata->state &= ~flag;
	article_update ( adata, sync );
}


/*---[ article_copy ]-------------------------------------------------
 * like a strdup function, returns an exact copy of the article passed
 * into it as a parameter
 *--------------------------------------------------------------------*/
article_data *
article_copy (article_data *src)
{
	article_data *dest;

	dest = (article_data *) g_malloc0 (sizeof(article_data));

	dest->number = g_strdup (src->number);
	dest->subject = g_strdup (src->subject);
	dest->author = g_strdup (src->author);
	dest->message_id = g_strdup (src->message_id);
	dest->references = g_strdup (src->references);
	dest->date = src->date;
	dest->linecount = src->linecount;
	dest->parts = src->parts;
	dest->part = src->part;
	dest->group = src->group;
	dest->header = g_strdup (src->header);
	dest->body = g_strdup (src->body);

	return dest;
}


/*--------------------------------------------------------------------
 * removes article from the database for that group, also removes it
 * from the displayed articlelist
 *
 * FIXME: deleting threads of a parent (and a parent) can cause
 *        a segfault.
 * FIXME: deleting the top-level parent node causes segfault.
 *--------------------------------------------------------------------*/
#if 0
void
article_delete (void)
{
	GList *list, *next;
	article_data *adata;
	group_data *gdata = g_malloc0 (sizeof (group_data));
	gint deleted = 0;
	gint read_deleted = 0;

	gtk_clist_freeze (GTK_CLIST(Pan.article_ctree));
	next = NULL;
	list = GTK_CLIST (Pan.article_ctree)->selection; 
	while (list) {
		next = list->next;
		adata = (article_data *) gtk_ctree_node_get_row_data (GTK_CTREE(Pan.article_ctree), list->data);
		if (adata) {
			ahdb_erase ( adata->group->article_header_db, adata );
			gdata = adata->group;
			gdata->articlelist = g_slist_remove (gdata->articlelist, adata);
			article_free (adata);
			--deleted;
			if (adata->state & STATE_READ)
				--read_deleted;
		}
		gtk_ctree_remove_node (GTK_CTREE(Pan.article_ctree), list->data);
		list = next;
	}
	gtk_clist_thaw (GTK_CLIST(Pan.article_ctree));

	group_add_attrib_i (gdata, "Total", deleted);
	group_add_attrib_i (gdata, "Read", read_deleted);
}
#endif


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/
void
article_delete_body (article_data *adata)
{
	if (adata->body)
		g_free (adata->body);

	adata->state &= ~STATE_DECODED;
}


int
article_find (article_data *adata, gchar *message_id)
{
	if (adata == NULL)
		return 1;

	return (strcmp (adata->message_id, message_id));
}


void
article_free (article_data *adata)
{
	g_free (adata->number);
	g_free (adata->subject);
	g_free (adata->author);
	g_free (adata->message_id);
	g_free (adata->references);
	g_free (adata->header);
	g_free (adata->body);
	g_free (adata);
}


/*---[ article_get_current ]------------------------------------------
 * returns the current displayed (or selected) article..
 *--------------------------------------------------------------------*/
article_data *
article_get_current (void)
{
	if (article)
		return article;
	else
		return NULL;
}



/*--------------------------------------------------------------------
 * should only be called from the network thread
 *--------------------------------------------------------------------*/
void
article_update (
	article_data* adata,
	gboolean sync )
{
	article_db db = NULL;

	/* entry assertions */
	g_assert ( adata && adata->message_id );

	/* update the database... */
	db = ahdb_ref ( Pan.active_server, Pan.active_group );
	ahdb_save ( db, adata, sync );
	ahdb_unref ( db );

	/* update the article ui... */
	pan_lock ();
	articlelist_update_node ( adata );
	pan_unlock ();
}


/*
 * Extracts the requested header from adata->header;
 * hdr is the case sensitive name, without the colon
 * Doesn't handle wrapped lines
 */
gchar *
extract_header (article_data *adata, gchar *hdr)
{
	gchar *start, *end, *header;

	header = g_strconcat (hdr, ": ", NULL);
	start = strstr (adata->header, header);
	g_free (header);
	if (start == NULL)
		return NULL;
	else {
		start = strchr (start, ':');
		end = strchr (start, '\n');
		return g_strndup (start + 2, end - start - 3);
	}
}

==> pan-0.7.0.tar.gz: pan-0.7.0/pan/article.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include "pan.h"

void
article_add_flag (const server_data *sdata, const group_data *gdata, article_data* adata, int flag, gboolean sync)
{
	adata->state |= flag;
	article_update ( sdata, gdata, adata, sync );
}


void
article_remove_flag (const server_data *sdata, const group_data *gdata, article_data* adata, int flag, gboolean sync)
{
	adata->state &= ~flag;
	article_update ( sdata, gdata, adata, sync );
}


/*---[ article_copy ]-------------------------------------------------
 * like a strdup function, returns an exact copy of the article passed
 * into it as a parameter
 *--------------------------------------------------------------------*/
article_data *
article_copy (const article_data *src)
{
	article_data *dest;

	dest = (article_data *) g_malloc0 (sizeof(article_data));

	dest->number = g_strdup (src->number);
	dest->subject = g_strdup (src->subject);
	dest->author = g_strdup (src->author);
	dest->message_id = g_strdup (src->message_id);
	dest->references = g_strdup (src->references);
	dest->date = src->date;
	dest->linecount = src->linecount;
	dest->parts = src->parts;
	dest->part = src->part;
	dest->followup_to = g_strdup ( src->followup_to );
	dest->reply_to = g_strdup ( src->reply_to );
	dest->newsgroups = g_strdup ( src->newsgroups );
	dest->state = src->state;

	//dest->group = group_copy (src->group);

	return dest;
}


void
article_free_gfunc(gpointer data, gpointer user_data)
{
	article_free ( (article_data*)data );
}
void
article_free (article_data *adata)
{
        g_return_if_fail (adata!=NULL);

	g_free (adata->number);
	g_free (adata->subject);
	g_free (adata->author);
	g_free (adata->message_id);
	g_free (adata->references);
	g_free (adata->newsgroups);
	g_free (adata->reply_to);
	g_free (adata->followup_to);
	g_free (adata);
}



/*--------------------------------------------------------------------
 * should only be called from the network thread
 *--------------------------------------------------------------------*/
void
article_update (const server_data *sdata, const group_data *gdata, article_data* adata, gboolean sync )
{
	article_db db = NULL;

	/* entry assertions */
	g_assert ( adata && adata->message_id );

	/* update the database... */
	db = ahdb_ref ( sdata, gdata );
	ahdb_save ( db, adata, sync );
	ahdb_unref ( db );

	/* update the article ui... */
	articlelist_update_node ( adata );
}


/*
 * Extracts the requested header from adata->header;
 * hdr is the case sensitive name, without the colon
 * Doesn't handle wrapped lines
 */
#if 0
gchar *
extract_header (article_data *adata, gchar *hdr)
{
	gchar* header = g_strconcat (hdr, ": ", NULL);
//	gchar* start = strstr (adata->header, header);
        gchar* end;
	g_free (header);
	if (start == NULL)
		return NULL;
	else {
		start = strchr (start, ':');
		end = strchr (start, '\n');
		return g_strndup (start + 2, end - start - 3);
	}
}
#endif
